@@ -4,8 +4,8 @@
 
 ;; Author: Patrick Thomson <patrickt@github.com>
 ;; URL: https://github.com/F4ban/codespaces.el
-;; Package-Version: 20260218.0
-;; Package-Revision: 542c029b38d7
+;; Package-Version: 20260224.403
+;; Package-Revision: 16e5fc290532
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: comm
 ;; Created: 2022-08-11
@@ -54,6 +54,10 @@ When this is nil, the default of '/workspaces/<repo-name>' is used."
   :group 'codespaces
   :type 'string)
 
+(defconst codespaces--null-device
+  (if (eq system-type 'windows-nt) "NUL" "/dev/null")
+  "The null device used for suppressing output, specific to the operating system.")
+
 (defvar codespaces--validated nil
   "Whether the gh CLI has been validated for codespaces access.")
 
@@ -71,9 +75,7 @@ This check is performed lazily on first use rather than at setup time."
     (let ((status
            (let ((inhibit-read-only t))
              (shell-command
-              (if (eq system-type 'windows-nt)
-                  "gh codespace list 1>NUL"
-                "gh codespace list 1>/dev/null")
+              (format "gh codespace list 1>%s" codespaces--null-device)
               nil "*codespaces-output*"))))
       (unless (zerop status)
         (user-error
@@ -227,8 +229,71 @@ allowing for faster startup.  Validation happens lazily on first use."
   (cl-loop for v being the hash-values of (codespaces--all-codespaces)
            collect (list nil (codespaces-space-name v))))
 
+(defun codespaces--get-default-branch (repo)
+  "Return the default branch for REPO or signal an error if not found."
+  (let* ((command (format "gh repo view %s --json defaultBranchRef -q .defaultBranchRef.name 2>%s"
+                         repo codespaces--null-device))
+         (branch (shell-command-to-string command)))
+    (if (string-empty-p branch)
+        (user-error "Error getting default branch.  Likely, no repository exists or it's inaccessible")
+      (string-trim branch))))
+
+(defun codespaces--create-from-args (repo branch machine)
+  "Create a codespace for args REPO BRANCH and MACHINE."
+  (message "Creating codespace...")
+  (let ((status
+         (codespaces--locally
+          (let ((inhibit-read-only t))
+          (shell-command
+           (format "gh codespace create --machine %s --repo %s --branch %s"
+                   machine repo branch)
+           nil "*codespaces-output*")))))
+    (if (zerop status)
+        (message "Codespace created successfully!")
+      (user-error "Command `gh codespace create` failed... [See *codespaces-output* buffer for details]"))))
+
+(defun codespaces--get-machine-types (repo)
+  "Return an alist of machine types for REPO with name as key."
+  (let* ((output (shell-command-to-string
+                  (format
+                   "gh api -H 'Accept: application/vnd.github+json' \
+-H 'X-GitHub-Api-Version: 2022-11-28' /repos/%s/codespaces/machines 2>%s"
+                   repo
+                   codespaces--null-device)))
+         (machines (json-read-from-string output))
+         (machine-list
+          (mapcar (lambda (machine)
+                    (cons (alist-get 'name machine)
+                          (alist-get 'display_name machine)))
+                  (alist-get 'machines machines))))
+    machine-list))
+
+(defun codespaces--select-machine (repo)
+  "Prompt user to select a machine type for REPO by display name."
+  (let* ((machine-alist (codespaces--get-machine-types repo))
+         (display-alist (mapcar (lambda (entry)
+                                  (cons (cdr entry) (car entry)))
+                                machine-alist))
+         (choice (completing-read
+                  "Select machine type: "
+                  display-alist
+                  nil t)))
+    (cdr (assoc choice display-alist))))
+
 ;;; Public interface
 
+(defun codespaces-create ()
+  "Prompt for repository, branch, and machine type to create a Codespace."
+  (interactive)
+  (codespaces--validate-gh)
+  (let* ((repo (read-string "Repository (user/repo): "))
+         (branch (read-string "Branch: " (or (codespaces--get-default-branch repo) "main")))
+         (machine (codespaces--select-machine repo)))
+    (when (string-empty-p repo)
+      (user-error "Repository must not be empty"))
+    (message "Creating codespace...")
+      (codespaces--create-from-args repo branch machine)))
+
 (defun codespaces-stop ()
   "Stop a codespace chosen by `completing-read'."
   (interactive)
